#include "stdafx.h"
#include "MakeHashAlgorithm.h"



////////////////////////////////////////////////////////////////////////////////////////////////////

CMakeHashAlgorithm::CMakeHashAlgorithm()
{
}

CMakeHashAlgorithm::~CMakeHashAlgorithm()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void CMakeHashAlgorithm::Initialize()
{
    m_qwSrcLen  = 0;
    m_dwRestLen = 0;
}

void CMakeHashAlgorithm::ComputeHash(const void *pBuffer, DWORD len)
{
    if ( len == 0 )
        return;

    DWORD tmp, restlen=m_dwRestLen;
    m_qwSrcLen += len;

    if ( restlen > 0 ) {
        DWORD dwHashBlockSize = GetHashBlockSize();
        if ( restlen+len < dwHashBlockSize ) {
            m_dwRestLen = restlen + len;
            memcpy(&m_pRest[restlen], pBuffer, len);
            return;
        } else {
            tmp = dwHashBlockSize - restlen;
            memcpy(&m_pRest[restlen], pBuffer, tmp);
            ComputeHashHelper(m_pRest, dwHashBlockSize);
            m_dwRestLen = 0;
            len -= tmp;
            pBuffer = (BYTE*)pBuffer + tmp;
            if ( len == 0 )
                return;
        }
    }

    restlen = ComputeHashHelper(pBuffer, len);
    if ( restlen > 0 ) {
        memcpy(m_pRest, (BYTE*)pBuffer+len-restlen, restlen);
        m_dwRestLen = restlen;
    }
}

void CMakeHashAlgorithm::GetHashValue(void *msg_digest)
{
    ComputeHashFinalHelper(msg_digest);
    Initialize();
}

void CMakeHashAlgorithm::GetHashValueWithoutInitializing(void *msg_digest)
{
    ComputeHashFinalHelper(msg_digest);
}

void CMakeHashAlgorithm::GetHashString(char *pStrBuf, BOOL bSmallString)
{
    static const char *xstring_s = "0123456789abcdef";
    static const char *xstring_b = "0123456789ABCDEF";
    const char *xstring = ( bSmallString ) ? xstring_s : xstring_b;
    DWORD i, size = GetHashValueSize();
    BYTE hash[1024];

    GetHashValue(hash);

    for (i=0; i<size; i++) {
        *(pStrBuf+0) = xstring[hash[i]>>4];
        *(pStrBuf+1) = xstring[hash[i]&0x0f];
        pStrBuf += 2;
    }
    *pStrBuf = '\0';
}


////////////////////////////////////////////////////////////////////////////////////////////////////
// ReverseEndian

#if (USE_REVERSEENDIAN_ASSEMBLER==0)

void ReverseEndian32(void *dest, const void *src, DWORD len)
{
    DWORD dwData;
    DWORD *pDest = (DWORD*)dest;
    DWORD *pSrc = (DWORD*)src;
    len /= 4;

    while ( len > 0 ) {
        dwData = *pSrc++;
        *pDest++ = (dwData << 24) | ((dwData & 0x0000ff00) << 8) | ((dwData & 0x00ff0000) >> 8) | (dwData >> 24);
        len--;
    }
}

void ReverseEndian64(void *dest, const void *src, DWORD len)
{
    DWORD dwData1, dwData2;
    DWORD *pDest = (DWORD*)dest;
    DWORD *pSrc = (DWORD*)src;
    len /= 8;

    while ( len > 0 ) {
        dwData1 = *pSrc;
        dwData2 = *(pSrc+1);
        *(pDest+1) = (dwData1 << 24) | ((dwData1 & 0x0000ff00) << 8) | ((dwData1 & 0x00ff0000) >> 8) | (dwData1 >> 24);
        *(pDest) = (dwData2 << 24) | ((dwData2 & 0x0000ff00) << 8) | ((dwData2 & 0x00ff0000) >> 8) | (dwData2 >> 24);
        pSrc+=2;
        pDest+=2;
        len--;
    }
}

#endif  // (USE_REVERSEENDIAN_ASSEMBLER==0)

////////////////////////////////////////////////////////////////////////////////////////////////////
